<?php

if (!defined('DS')) {
    define('DS', DIRECTORY_SEPARATOR);
}

$whmcs_dir_path = dirname(__FILE__) . DS . '..';

if (file_exists($whmcs_dir_path . DS . 'init.php')) {
    require_once $whmcs_dir_path . DS . 'init.php';
} else {
    require_once $whmcs_dir_path . DS . 'dbconnect.php';
    require_once $whmcs_dir_path . DS . 'includes' . DS . 'functions.php';
}

require_once $whmcs_dir_path . DS . 'modules' . DS . 'addons' . DS . 'HostwindsReseller' . DS . 'core' . DS . 'functions.php';
require_once $whmcs_dir_path . DS . 'HostwindsResellerAPI' . DS . 'classes' . DS . 'class.HostwindsReseller_Hosting.php';
require_once $whmcs_dir_path . DS . 'HostwindsResellerAPI' . DS . 'helpers' . DS . 'WHMCS.php';
ob_clean();

if (!class_exists('MG_ApiController')) {

    class MG_ApiController
    {
        private static $_errorMessages = [
            'NO_API_KEY' => 'No API KEY in request',
            'NO_EMAIL' => 'No reseller email in request',
            'INVALID_API_KEY' => 'API KEY is invalid',
            'INVALID_EMAIL' => 'Email address is invalid',
            'PARAMS_NO_ACTION' => 'Params error - no action',
            'PARAMS_UNKNOWN_ACTION' => 'Params error - unknown action',
            'PARAMS_NOT_ACTIVE' => 'This service\'s status does not allow to manage it',
            'NO_PARAM' => 'No param - ',
        ];
        private $params_in;

        public function __construct($params)
        {
            $this->params_in = $params;
            //throw new Exception(print_r($params, true));
        }

        public function processRequest()
        {
            $api = unserialize(decrypt(getWHMCSconfig('HostwindsResellerServerConfiguration')));
            // NO_ACTION
            if (!isset($this->params_in['action'])) {
                $this->sendErrorResponse(self::$_errorMessages['PARAMS_NO_ACTION']);
            }
            // NO_API_KEY
            if (!isset($this->params_in['reseller_api_key'])) {
                $this->sendErrorResponse(self::$_errorMessages['NO_API_KEY']);
            }
            // NO_EMAIL
            if (!isset($this->params_in['reseller_email'])) {
                $this->sendErrorResponse(self::$_errorMessages['NO_EMAIL']);
            }

            try {
                $this->checkReseller($api);
            } catch (Exception $e) {
                if ($e->getMessage() == 'INVALID_API_KEY') {
                    $this->sendErrorResponse(self::$_errorMessages['INVALID_API_KEY']);
                }
                if ($e->getMessage() == 'INVALID_EMAIL') {
                    $this->sendErrorResponse(self::$_errorMessages['INVALID_EMAIL']);
                }
                throw $e;
            }

            if ($this->params_in['action'] == 'SendWelcomeEmail') {
                $this->_call_SendWelcomeEmail();
            }
        }

        protected function _call_SendWelcomeEmail()
        {
            if (!isset($this->params_in['hosting_id'])) {
                $this->sendErrorResponse(self::$_errorMessages['NO_PARAM'] . 'hosting_id');
            }

            $hosting = new HostwindsReseller_Hosting($this->params_in['hosting_id']);
            $res = $hosting->sendWelcomeEmail();
            $this->sendSuccessResponse($res['result']);
        }

        private function checkReseller(array $api)
        {
            if ($api['password'] != $this->params_in['reseller_api_key']) {
                throw new Exception('INVALID_API_KEY');
            }

            if ($api['email'] != $this->params_in['reseller_email']) {
                throw new Exception('INVALID_EMAIL');
            }
        }

        private function sendErrorResponse($error, array $params = [])
        {
            $res = [
                'result' => 0,
                'msg' => $error,
            ];
            $str = json_encode(array_merge($res, $params));
            header('Content-Type: application/json');
            header('Content-Length: ' . strlen($str));
            echo $str;
            die();
        }

        private function sendSuccessResponse($info, $params = [])
        {
            $res = [
                'result' => 1,
                'msg' => $info,
            ];
            $str = json_encode(array_merge($res, $params));
            header('Content-Type: application/json');
            header('Content-Length: ' . strlen($str));
            echo $str;
            die();
        }
    }
}

try {
    $api = new MG_ApiController($_POST);
    $api->processRequest();
} catch (Exception $e) {
    die(json_encode([
        'result' => 0,
        'msg' => $e->getMessage(),
    ]));
}
