<?php

class API
{
    private $params;

    private $timeout = 30;

    public function __construct()
    {
        $this->setApiParams($this->getParams());
    }

    private function setApiParams($params = [])
    {
        $this->params = $params;
    }

    private function getParams()
    {
        $params = [];
        $api = unserialize(decrypt(getWHMCSconfig('HostwindsResellerServerConfiguration')));
        if (!empty($api)) {
            $params['serverhostname'] = $api['hostname'];
            $params['serverusername'] = $api['email'];
            $params['serverpassword'] = $api['password'];
        }

        return $params;
    }

    public function testConnection($apiParams)
    {
        return $this->callApi('TestConnection', [], $apiParams);
    }

    public function getProductsConfigOptions($ids)
    {
        return $this->callApi('AllProductsWithConfigurableOptions', ['product_id' => $ids]);
    }

    public function getSyncWeekDay()
    {
        return $this->callApi('GetSyncWeekDay');
    }

    public function getServicesData($hostingIDs, $params)
    {
        return $this->callApi('GetServicesData', ['hostings_ids' => $hostingIDs, 'paramsToSync' => $params]);
    }

    public function getProductsAddons($ids)
    {
        return $this->callApi('ProductsAddons', ['product_id' => $ids]);
    }

    public function sendResellerApiUrl($url)
    {
        return $this->callApi('SaveResellersApiUrl', ['url' => $url]);
    }

    private function callApi($action, array $postfields = [], $apiParams = [])
    {
        if (!$this->params['serverhostname'] || !$this->params['serverusername'] || !$this->params['serverpassword']) {
            $this->setApiParams($this->getParams());
        }
        //get api params from api detalis form
        if (!empty($apiParams)) {
            $this->setApiParams($apiParams);
        }

        $uri = 'Hostwinds';
        $uri .= 'ResellerAPI';

        if (strpos($this->params['serverhostname'], '/' . $uri . '/api.php') === false) {
            $this->params['serverhostname'] = $this->params['serverhostname'] . '/' . $uri . '/api.php';
        }

        if (!$this->params['serverhostname'] || !$this->params['serverusername'] || !$this->params['serverpassword']) {
            throw new Exception('API details are not configured - check fields hostname, e-mail and API key', 500);
        }

        $postfields["reseller_api_key"] = $this->params['serverpassword'];
        $postfields["reseller_email"] = $this->params['serverusername'];
        $postfields["action"] = $action;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->params['serverhostname']);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postfields));

        $data = curl_exec($ch);

        if (!$data) {
            $this->error = curl_error($ch);
            $e = $this->error ? 'cURL error: ' . $this->error : 'EMPTY RESPONSE';
            logModuleCall($action, $postfields, $e);
            throw new Exception($e, 500);
        }
        switch (curl_getinfo($ch, CURLINFO_HTTP_CODE)) {
            case 404:
                $e = 'The request hostname is invalid.';
                logModuleCall($action, $postfields, $e);
                throw new Exception($e, 500);
        }

        curl_close($ch);

        return $response = json_decode($data, true);

    }
}
