<?php

require_once ADDON_DIR . DS . 'classes' . DS . 'Products.php';
require_once ADDON_DIR . DS . 'classes' . DS . 'API.php';

class ConfigOptionsSync
{
    private $products = [];
    private $targetProducts = [];

    public function __construct($id = null)
    {
        $products = new Products($id);
        //get reseller's product IDs
        $this->products = $products->getProducts();
        //get target product IDs
        $this->targetProducts = $products->getTargetProducts();
    }

    public function configOptionsSync($multiplier = '1.00')
    {
        //prepate product target IDs
        $ids = [];
        foreach ($this->targetProducts as $targetProduct) {
            if (!in_array($targetProduct['taget_product_id'], $ids)) {
                $ids[] = $targetProduct['taget_product_id'];
            }
        }

        //send request with product target IDs
        $api = new API();
        $response = $api->getProductsConfigOptions($ids);
        $configOptionsGroups = [];
        foreach ($response['products'] as $targetProductID => $product) {
            $configurableOptions = [];
            foreach ($this->targetProducts as $relation) {
                if ($relation['taget_product_id'] == $targetProductID) {
                    $productID = $relation['product_id'];

                    foreach ($product['configurable_options'] as $key => $configOption) {
                        if (isset($prodCOGroup) && $configOption['groupname'] != $prodCOGroup['title']) {
                            $prodCOGroup = [];
                        }

                        $prodCOGroup['title'] = $configOption['groupname'];
                        $prodCOGroup['description'] = $configOption['description'];

                        $optionName = $configOption['optionname'];
                        $prodCOGroup['fields'][$optionName] = [
                            'type' => $configOption['optiontype'],
                            'order' => $configOption['order'],
                            'qtyminimum' => $configOption['qtyminimum'],
                            'qtymaximum' => $configOption['qtymaximum'],
                            'optionname' => $optionName,
                            'options' => [],
                        ];

                        $currency_id = null;
                        // currency code to id
                        if (isset($configOption['options'][0]['currency'])) {
                            $rowcurr = mysql_extract_assoc(mysql_query_safe('SELECT id FROM tblcurrencies WHERE code = ?',
                                [$configOption['options'][0]['currency']]));
                            $currency_id = isset($rowcurr['id']) ? $rowcurr['id'] : null;
                        }

                        foreach ($configOption['options'] as $o) {
                            $prodCOGroup['fields'][$optionName]['options'][] = [
                                'optionname' => $o['optionname'],
                                'sortorder' => $o['sortorder'],
                                'pricing' => !$currency_id ? [] : [
                                    $currency_id => [
                                        'msetupfee' => $o['msetupfee'],
                                        'qsetupfee' => $o['qsetupfee'],
                                        'ssetupfee' => $o['ssetupfee'],
                                        'asetupfee' => $o['asetupfee'],
                                        'bsetupfee' => $o['bsetupfee'],
                                        'tsetupfee' => $o['tsetupfee'],
                                        'monthly' => $o['monthly'],
                                        'quarterly' => $o['quarterly'],
                                        'semiannually' => $o['semiannually'],
                                        'annually' => $o['annually'],
                                        'biennially' => $o['biennially'],
                                        'triennially' => $o['triennially'],
                                    ],
                                ],
                            ];
                        }
                        $configurableOptions[$configOption['groupname']] = $prodCOGroup;
                    }
                    $configOptionsGroups[$productID] = $configurableOptions;
                }
            }
        }
        //generate config options
        $this->generateConfigurableOptions($configOptionsGroups, $multiplier);
    }

    private function generateConfigurableOptions($configOptions, $multiplier)
    {
        foreach ($configOptions as $productID => $productGroups) {
            $assignedConfigOptionsGropus = $this->getAssignedConfigOptionsGroups($productID);

            foreach ($productGroups as $group) {
                //chack if config option group exist
                $existingGroup = $this->existConfigurableOptionsGroup($group['title']);
                //if not exist create group
                if ($existingGroup == false) {
                    mysql_query_safe('INSERT INTO tblproductconfiggroups(name,description) VALUES(?, ?)', [
                        $group['title'],
                        $group['description'],
                    ]);
                    $group_id = mysql_last_id();
                } else {
                    $group_id = $existingGroup['id'];
                    mysql_query_safe('UPDATE tblproductconfiggroups SET description = ? WHERE id = ?', [
                        $group['description'],
                        $group_id,
                    ]);
                }

                //check if config option group is assigned to product
                $hasGroup = $this->hasConfigurableOptionsGroup($productID, $group_id);
                //assign  config option group if not assigned
                if ($hasGroup == false) {
                    mysql_query_safe('INSERT INTO tblproductconfiglinks(gid,pid) VALUES(?,?)', [$group_id, $productID]);
                }

                //get existing config options
                $existingConfigOptions = $this->getExistingConfigurableOptions($group_id);

                $doNotHide = [];
                foreach ($group['fields'] as $field_key => $field) {
                    switch ($field['type']) {
                        case 'select':
                        case 1:
                        case 'dropdown':
                            $field_type = 1;
                            break;
                        case 'radio':
                        case 2:
                            $field_type = 2;
                            break;
                        case 'yesno':
                        case 3:
                            $field_type = 3;
                            break;
                        case 'quantity':
                        case 4:
                            $field_type = 4;
                            break;
                        default:
                            continue;
                    }

                    $existingOption = $this->hasConfigurableOption($existingConfigOptions, $group_id,
                        $field['optionname'], $field_type);

                    if ($existingOption == false) {
                        mysql_query_safe('INSERT INTO tblproductconfigoptions(gid,optionname,optiontype,qtyminimum,qtymaximum,`order`,hidden) VALUES(?,?,?,?,?,?,0)',
                            [
                                $group_id,
                                $field['optionname'],
                                $field_type,
                                isset($field['qtyminimum']) ? (int)$field['qtyminimum'] : 0,
                                isset($field['qtymaximum']) ? (int)$field['qtymaximum'] : 0,
                                isset($field['order']) ? (int)$field['order'] : 0,
                            ]);

                        $config_id = mysql_last_id();
                    } elseif (!is_array($existingOption) && $existingOption != false) {
                        //do not nothing if config option type will change in main whmcs
                        //do not hide groups and update
                        $doNotHide[] = $existingOption;
                        continue;
                    } else {
                        $config_id = $existingOption['id'];
                        mysql_query_safe('UPDATE tblproductconfigoptions SET qtyminimum = ?, qtymaximum = ?, hidden = 0 WHERE id = ?',
                            [
                                isset($field['qtyminimum']) ? (int)$field['qtyminimum'] : 0,
                                isset($field['qtymaximum']) ? (int)$field['qtymaximum'] : 0,
                                $config_id,
                            ]);
                    }

                    $existingConfigOptionsSubs = $this->getExistingConfigurableOptionSubs($config_id);

                    foreach ($field['options'] as $option) {
                        $existingOptionSub = $this->hasConfigurableOptionSub($config_id, $option['optionname']);
                        if ($existingOptionSub == false) {
                            //Insert options
                            mysql_query_safe("INSERT INTO tblproductconfigoptionssub(configid,optionname,hidden) VALUES(?,?,?)",
                                [
                                    $config_id,
                                    $option['optionname'],
                                    isset($field['hidden']) && (int)$field['hidden'] == 1 ? 1 : 0,
                                ]);
                            $suboption_id = mysql_last_id();

                            if (isset($option['pricing'])) {
                                foreach ($option['pricing'] as $currency_id => $values) {
                                    mysql_query_safe('INSERT INTO `tblpricing` (`type`,`currency`,`relid`,`msetupfee`,`qsetupfee`,`ssetupfee`,`asetupfee`,`bsetupfee`,`tsetupfee`,`monthly`,`quarterly`,`semiannually`,`annually`,`biennially`,`triennially`) VALUES("configoptions",?,?,?,?,?,?,?,?,?,?,?,?,?,?)',
                                        [
                                            $currency_id,
                                            $suboption_id,
                                            in_array($values['msetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['msetupfee'] : $values['msetupfee'] * $multiplier,
                                            in_array($values['qsetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['qsetupfee'] : $values['qsetupfee'] * $multiplier,
                                            in_array($values['ssetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['ssetupfee'] : $values['ssetupfee'] * $multiplier,
                                            in_array($values['asetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['asetupfee'] : $values['asetupfee'] * $multiplier,
                                            in_array($values['bsetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['bsetupfee'] : $values['bsetupfee'] * $multiplier,
                                            in_array($values['tsetupfee'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['tsetupfee'] : $values['tsetupfee'] * $multiplier,
                                            in_array($values['monthly'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['monthly'] : $values['monthly'] * $multiplier,
                                            in_array($values['quarterly'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['quarterly'] : $values['quarterly'] * $multiplier,
                                            in_array($values['semiannually'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['semiannually'] : $values['semiannually'] * $multiplier,
                                            in_array($values['annually'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['annually'] : $values['annually'] * $multiplier,
                                            in_array($values['biennially'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['biennially'] : $values['biennially'] * $multiplier,
                                            in_array($values['triennially'], [
                                                '-1.00',
                                                '0.00',
                                            ]) ? $values['triennially'] : $values['triennially'] * $multiplier,
                                        ]);
                                }
                            }
                        } else {
                            $this->unhideConfigOptionSub($existingOptionSub['id']);
                        }
                        //prepare config option subs to hide
                        foreach ($existingConfigOptionsSubs as $key => $configOptionsSub) {
                            if ($configOptionsSub['optionname'] == $option['optionname']) {
                                unset($existingConfigOptionsSubs[$key]);
                            }
                        }
                    }
                    //hide config option sub if not exist on main whmcs
                    foreach ($existingConfigOptionsSubs as $configOptionSub) {
                        $this->hideConfigOptionSub($configOptionSub['id']);
                    }

                    //prepare config option to hide
                    foreach ($existingConfigOptions as $key => $configOption) {
                        if ($configOption['optionname'] == $field['optionname']) {
                            unset($existingConfigOptions[$key]);
                        }
                    }
                }
                //hide config option if not exist on main whmcs
                foreach ($existingConfigOptions as $configOption) {
                    if (!in_array($configOption['id'], $doNotHide)) {
                        $this->hideConfigOption($configOption['id']);
                    }
                }
                //prepare array of config options gropups to unassing from product
                foreach ($assignedConfigOptionsGropus as $key => $gorupID) {
                    if ($gorupID == $group_id) {
                        unset($assignedConfigOptionsGropus[$key]);
                    }
                }
            }
            foreach ($assignedConfigOptionsGropus as $groupID) {
                $this->unassignConfigOptionsGroupFromProduct($productID, $groupID);
            }
        }
    }

    private function unhideConfigOptionSub($id)
    {
        mysql_query_safe('UPDATE tblproductconfigoptionssub SET hidden = ? WHERE id = ? AND hidden = 1', [0, $id]);
    }

    private function hideConfigOptionSub($id)
    {
        mysql_query_safe('UPDATE tblproductconfigoptionssub SET hidden = ? WHERE id = ?', [1, $id]);
    }

    private function hideConfigOption($id)
    {
        mysql_query_safe('UPDATE tblproductconfigoptions SET hidden = ? WHERE id = ?', [1, $id]);
    }

    private function unassignConfigOptionsGroupFromProduct($pid, $gid)
    {
        mysql_query_safe('DELETE FROM tblproductconfiglinks WHERE gid = ? AND pid = ?', [$gid, $pid]);
    }

    private function existConfigurableOptionsGroup($groupName)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfiggroups WHERE name = ?', [$groupName]);

        return mysql_extract_assoc($q);
    }

    private function hasConfigurableOptionsGroup($pid, $gid)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfiglinks WHERE pid = ? AND gid = ?', [$pid, $gid]);

        return mysql_extract_assoc($q);
    }

    private function hasConfigurableOption($options, $gid, $optionName, $optionType)
    {
        foreach ($options as $configOption) {
            if ($configOption['gid'] == $gid && $configOption['optionname'] == $optionName) {
                if ($configOption['optiontype'] == $optionType) {
                    return $configOption;
                } else {
                    //retrun id and do not nothing if config option type will change in main whmcs
                    return $configOption['id'];
                }
            }
        }

        return false;
    }

    private function hasConfigurableOptionSub($configid, $optionName)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfigoptionssub WHERE configid = ? AND optionname = ?',
            [$configid, $optionName]);

        return mysql_extract_assoc($q);
    }

    private function getExistingConfigurableOptionSubs($configid)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfigoptionssub WHERE configid = ?', [$configid]);

        $configOptionSubs = [];
        while ($row = mysql_extract_assoc($q)) {
            $configOptionSubs[] = $row;
        }
        return $configOptionSubs;
    }

    private function getExistingConfigurableOptions($gid)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfigoptions WHERE gid = ?', [$gid]);

        $configOptions = [];
        while ($row = mysql_extract_assoc($q)) {
            $configOptions[] = $row;
        }
        return $configOptions;
    }

    private function getAssignedConfigOptionsGroups($pid)
    {
        $q = mysql_query_safe('SELECT * FROM tblproductconfiglinks WHERE pid = ?', [$pid]);

        $assignedGroups = [];
        while ($row = mysql_extract_assoc($q)) {
            $assignedGroups[] = $row['gid'];
        }
        return $assignedGroups;
    }
}
