<?php

abstract class HostwindsReseller_default_module
{

    public $params;
    public $product;
    public $hosting;
    public $defaultCustomField = [];

    public $timeout = 30;

    public abstract function CreateAccount();

    public function getProductsFromSeller()
    {

        return $this->callApi('ProductsList');
    }


    public function getProductDetailsForCreation($prod_id)
    {
        return $this->callApi('ProductDetailsForCreation', [
            'product_id' => $prod_id,
        ]);
    }

    public function rerouteTicket($subject, $content)
    {
        return $this->callApi('rerouteTicket', [
            'subject' => $subject,
            'content' => $content,
        ]);
    }

    public function testConnection()
    {
        $this->timeout = 5;
        return $this->callApi('TestConnection');
    }

    protected function callApi($action, array $postfields = [])
    {
        //if (!$this->params['serverhostname'] || !$this->params['serverusername'] || !$this->params['serverpassword'])
        $this->params = $this->product->getParams();

        $uri = 'Hostwinds';
        $uri .= 'ResellerAPI';

        if (strpos($this->params['serverhostname'], '/' . $uri . '/api.php') === false) {
            $this->params['serverhostname'] = $this->params['serverhostname'] . '/' . $uri . '/api.php';
        }

        if (!$this->params['serverhostname'] || !$this->params['serverusername'] || !$this->params['serverpassword']) {
            throw new Exception('API details are not configured - check fields hostname, e-mail and API key', 500);
        }

        $postfields["reseller_api_key"] = $this->params['serverpassword'];
        $postfields["reseller_email"] = $this->params['serverusername'];
        $postfields["action"] = $action;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->params['serverhostname']);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postfields));
        $data = curl_exec($ch);

        if (!$data) {
            $this->error = curl_error($ch);
            $e = $this->error ? 'cURL error: ' . $this->error : 'EMPTY RESPONSE';
            if ($this->product->getConfig('debug_mode')) {
                $this->logModuleCall($action, $postfields, $e);
            }
            throw new Exception($e, 500);
        }
        curl_close($ch);

        if ($this->product->getConfig('debug_mode')) {
            $this->logModuleCall($action, $postfields, $data);
        }

        $response = json_decode($data, true);
        if (!is_array($response)) {
            throw new Exception('API does not return JSON response', 500);
        }

        return new HostwindsReseller_default_ApiResponse($response);
    }

    protected function logModuleCall($action, array $params, $response)
    {
        if (function_exists('logModuleCall')) {
            // hide some values
            foreach (['reseller_api_key', 'rootpw'] as $key) {
                if (isset($params[$key])) {
                    $params[$key] = substr($params[$key], 0, 3) . '...';
                }
            }
            return logModuleCall(
                get_class($this),
                $action,
                'POST ' . $this->params['serverhostname'] . "\n" . print_r($params, true),
                '',
                $response
            );
        }
    }

    public function getModuleSettings()
    {
        return [];
    }

    public function getAjax()
    {
        $submodule = substr(get_class($this), strlen('HostwindsReseller_default_'));
        $file = dirname(__FILE__) . DS . $submodule . DS . 'ajax.php';
        if (file_exists($file)) {
            require_once $file;
            $cl = 'HostwindsReseller_default_' . $submodule . '_ajax';
            if (class_exists($cl)) {
                return new $cl();
            }
        }
        throw new Exception('No Ajax class');
    }

    public function getLang($lang = null)
    {
        global $CONFIG;
        $submodule = substr(get_class($this), strlen('HostwindsReseller_default_'));
        $language_dir = dirname(__FILE__) . DS . $submodule . DS . 'langs' . DS;
        $lang_requested = $lang ? $lang : (isset($_SESSION['Language']) ? $_SESSION['Language'] : $CONFIG['Language']);

        $language = $lang_requested && file_exists($language_dir . $lang_requested . '.php') ? $lang_requested : 'english';

        if (!file_exists($language_dir . $language . '.php')) {
            return [];
        }

        require_once $language_dir . $language . '.php';
        return isset($_LANG) ? $_LANG : [];
    }

    public function getAjaxUrl($action, $hosting_id = null, $isAdminArea = false)
    {
        return ($isAdminArea ? 'index' : 'clientarea') . '.php?HostwindsReseller_defaultAjax=1&action=' . $action . ($hosting_id ? '&hosting_id=' . $hosting_id : '');
    }

    public function getClientareaUrl($submodulepage, $hosting_id)
    {
        return 'clientarea.php?action=productdetails&id=' . $hosting_id . '&modop=custom&a=manage&submodulepage=' . $submodulepage;
    }

    /**
     *  Generate Default Custom Fieds
     * @return boolean
     */
    public function generateDefaultCustomField($prod_id)
    {
        foreach ($this->defaultCustomField as $key => $field) {
            $q = mysql_query_safe("SELECT id, relid, fieldname
			FROM tblcustomfields
			WHERE
				relid = " . $prod_id . "
				AND type = 'product'
				AND (fieldname = '" . $key . "' OR fieldname LIKE '" . $key . "|%') ");

            if (mysql_count_rows($q)) {
                return false;
            }

            switch ($field['type']) {
                case 'text':
                    mysql_query_safe('INSERT INTO tblcustomfields(type,relid,fieldname,fieldtype,description,fieldoptions,regexpr,adminonly,required,showorder,showinvoice,sortorder)
			VALUES("product", ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', [
                        $prod_id,
                        $key . '|' . $field['title'],
                        "text",
                        $field['description'] ? $field['description'] : '',
                        $field['fieldoptions'] ? $field['fieldoptions'] : '',
                        $field['regexpr'] ? $field['regexpr'] : '',
                        $field['adminonly'] ? 'on' : '',
                        $field['required'] ? 'on' : '',
                        $field['showorder'] ? 'on' : '',
                        $field['showinvoice'] ? 'on' : '',
                        $field['sortorder'] ? ' on' : '',
                    ]);
                    break;
                case 'quantity':
                    mysql_query_safe('INSERT INTO tblproductconfigoptions(gid,optionname,optiontype,qtyminimum,qtymaximum,`order`,hidden) VALUES(?,?,4,?,?,0,0)',
                        [
                            $group_id,
                            $field_key . '|' . $field['title'],
                            $field['min'],
                            $field['max'],
                        ]);
                    break;
                default:
                    die('Unsupported type!');
            }
        }

        return true;
    }

    /**
     * Helper method, you can use it in function AdminServicesTabFields
     * @param string $name Friendly name
     * @param string $ajaxaction Name of method in your ajax.php file e.g. ReissueLicense for method call_ReissueLicense
     * @param int $serviceid
     * @return string
     */
    protected function getCommandButtonHtml($name, $ajaxaction, $serviceid)
    {
        $id = md5(uniqid($name, true));
        return '
			<button class="btn" id="' . $id . '">' . $name . '</button>
			<script type="text/javascript">
			$(document).ready(function(){
				$("#' . $id . '").click(function(){
					$(this).addClass("disabled");
					$.post("' . $this->getAjaxUrl($ajaxaction, $serviceid, true) . '", {}, function(res){
						$("#' . $id . '").after("<strong style=\""+(res.success== 1 ? ";green;" : ";red;")+";\">"+res.msg+"</strong>";)
						$("#' . $id . '").remove();
					}, "json");
					return false;
				});
			});
			</script>
		';
    }

    protected function _redirect($url)
    {
        ob_clean();
        header('Location: ' . $url);
        die();
    }

    public function __call($name, $arguments)
    {
        throw new Exception('Module method /' . $name . '/ is not implemented', 500);
    }


    /**
     * DO NOT INITIAL MODULE THE OTHER WAY
     *
     * @param mixed $product int|HostwindsReseller_default_ResellerProduct
     * @return HostwindsReseller_default_module
     * @throws Exception
     */
    public static function getModule($product)
    {
        if (is_numeric($product)) { // Product ID
            $pr = new HostwindsReseller_default_ResellerProduct($product);
        } elseif ($product instanceof HostwindsReseller_default_ResellerProduct) {
            $pr = $product;
        } else {
            throw new Exception('Unknown product', 500);
        }

        // @dynamicsubmodules
        // $submodule = $pr->getConfig('submodule') ? $pr->getConfig('submodule') : 'default';

        // first submodule
        $allSubmodules = self::getAllModules();
        $submodule = $allSubmodules[0];

        $module = 'HostwindsReseller_default_' . $submodule;
        if (!class_exists($module)) {
            $path = dirname(__FILE__) . DS . $submodule . DS . $submodule . '.php';
            if (!file_exists($path)) {
                throw new Exception('Module file not found', 403);
            }
            require_once $path;
        }
        if (!class_exists($module)) {
            throw new Exception('Submodule class ' . $module . ' not found', 500);
        }

        $m = new $module();

        if (!($m instanceof HostwindsReseller_default_module)) {
            throw new Exception('Module does not extend HostwindsReseller_default_module', 500);
        }

        try {
            $m->product = $pr;
            $m->product->load();
        } catch (Exception $e) {

        }

        return $m;
    }

    public static function getAllModules()
    {
        $modules = [];
        $path = dirname(__FILE__) . DS;
        if ($handle = opendir($path)) {
            while (false !== ($entry = readdir($handle))) {
                if ($entry != "." && $entry != ".." && is_dir($path . $entry) && file_exists($path . $entry . DS . $entry . '.php')) {
                    $modules[] = $entry;
                }
            }
            closedir($handle);
        }
        return $modules;
    }


}
